#!/usr/bin/perl 
#
# part of Weathermanage v1.0
# 
# Mathieu Gilardet 
# released under the GNU Public License
#
# script used by weathermanage.php 
# for quering devices about its networking hardware
#
# usage: ifbrowse.pl <target-host> <interface-name-to-match> [community] [version]


use SNMP '5.0.2.pre1' || die("include\n");

$host = $ARGV[0];
if (!"$host") {
    print "usage\n";
    exit; 
}

$pattern = $ARGV[1];
if (!"$pattern") {
    print "usage\n";
    exit; 
}

$community = $ARGV[2];
if (!"$community") {
    $community = "public";
}

$version = $ARGV[3];
if (!"$version") {
    $version = "2c";
}

if ($version != "1" && $version != "2" && $version != "2c") {
    print "version\n";
    exit; 
}



# create an SNMP object 
$sess = new SNMP::Session(
			   'DestHost' => $host,
			   'Community' => $community,
			   'RemotePort'=> 161,
			   'Timeout'	=> 1000000,
			   'Retries'	=> 2,
			   'Version'	=> $version,
			   'UseLongNames' => 1,	   
			   'UseNumeric' => 0,	 
			   'UseEnums'=> 0,	 
			   'UseSprintValue' => 1
			   ); 


# get number of interfaces 
my $var_ifnumber = new SNMP::VarList(['IF-MIB::ifNumber.0']); 
$ifnumber = $sess->get($var_ifnumber);
if ($sess->{ErrorNum}) {
    print "connection_if\n"; 
    exit; 
}
if (!"$ifnumber") {
    print "connection_if\n"; 
    exit; 
}


# now browse them until matching the pattern (patern is iface name)
$found = 0; 
$i = 1; 
while (!$found && $i <= $ifnumber) {


    # cicso systems use ifName for naming interfaces, 
    # linux systems use ifDescr. 
    # default is ifName, and if not implemented, ifDescr will be used. 
    #
    # try to find a Linux and if not found use ifName. But checks for erros messages too. 
    # if any, use ifDescr as a Linux. 
    my $var_ifname = new SNMP::VarList(
				       ["SNMPv2-MIB::sysDescr.0"],
				       ["IF-MIB::ifName.$i"],
				       ); 
    @ifname = $sess->get($var_ifname);
    
    
    $os = $ifname[0]; 
    $name = $ifname[1];
    

    # a Linux response 
    if ($name eq "No Such Object available on this agent at this OID" || $name eq "Wrong Type (should be OCTET STRING): NULL" || $os =~ /.*linux.*/i) { 
	my $var_ifname = new SNMP::VarList(
					   ["IF-MIB::ifDescr.$i"],
					   ); 
	@ifname = $sess->get($var_ifname);
	$name = $ifname[0]; 
    }	
 
    if ($name eq $pattern) {
	$found = 1; 
    }
    else {
	$i++; 
    }

}

if (!$found) {
    print "notfound\n"; 
    exit; 
}


# finaly get speed 
my $var_ifname = new SNMP::VarList(["IF-MIB::ifSpeed.$i"]); 
@ifspeed = $sess->get($var_ifname);

# if speed overruns oid ifSpeed object (an ATM link case), try with ifHighSpeed
if ($ifspeed[0] == 4294967295) {
    
    my $var_ifdetails = new SNMP::VarList(["IF-MIB::ifHighSpeed.$i"]); 
    @speed = $sess->get($var_ifdetails);

    # check if valid response (numerical)
    if ($speed[0] =~ /^[0-9]*$/) { 	 
	$ifspeed[0] = @speed[0] * 1000000; 
    }	
}

print "$ifspeed[0]\n"; 



