<?php
/*******************************************************************************

    Author ......... Howard Jones
    Contact ........ howie@thingy.com
    Home Site ...... http://wotsit.thingy.com/haj/
    Program ........ Network Weathermap for Cacti
    Version ........ See code below
    Purpose ........ Network Usage Overview

*******************************************************************************/

function weathermap_version () {
        return array( 	'name'    	=> 'weathermap',
                        'version'       => '0.81',
                        'longname'      => 'PHP Network Weathermap',
                        'author'        => 'Howard Jones',
                        'homepage'      => 'http://wotsit.thingy.com/haj/cacti/php-weathermap/',
                        'email' 	=> 'howie@thingy.com',
                        'url'           => 'http://wotsit.thingy.com/haj/cacti/versions.php'
                        );
}

function plugin_init_weathermap() {
	global $plugin_hooks;
	$plugin_hooks['top_header_tabs']['weathermap'] = 'weathermap_show_tab';
        $plugin_hooks['top_graph_header_tabs']['weathermap'] = 'weathermap_show_tab';

	$plugin_hooks['config_arrays']['weathermap'] = 'weathermap_config_arrays';
        $plugin_hooks['draw_navigation_text']['weathermap'] = 'weathermap_draw_navigation_text';
	$plugin_hooks['config_settings']['weathermap'] = 'weathermap_config_settings';
        $plugin_hooks['poller_bottom']['weathermap'] = 'weathermap_poller_bottom';

	$plugin_hooks['top_graph_refresh']['weathermap'] = 'weathermap_top_graph_refresh';
}

function weathermap_top_graph_refresh($refresh)
{
	if (basename($_SERVER["PHP_SELF"]) != "weathermap-cacti-plugin.php")
		return $refresh;
	
	// if we're cycling maps, then we want to handle reloads ourselves, thanks
	if($_REQUEST["action"] == 'viewmapcycle')
	{
		return(86400);
	}
	return ($refresh);
}

function weathermap_config_settings () {
	global $tabs, $settings;
	$tabs["misc"] = "Misc";
	
	$temp = array(
		"weathermap_header" => array(
			"friendly_name" => "Network Weathermap",
			"method" => "spacer",
			),
		"weathermap_pagestyle" => array(
			"friendly_name" => "Page style",
			"description" => "How to display multiple maps.",
			"method" => "drop_array",
			"array" => array(0 => "Thumbnail Overview", 1 => "Full Images")
			),
		"weathermap_thumbsize" => array(
			"friendly_name" => "Thumbnail Maximum Size",
			"description" => "The maximum width or height for thumbnails in thumbnail view, in pixels. Takes effect after the next poller run.",
			"method" => "textbox",
			"max_length" => 5,
			),
		"weathermap_cycle_refresh" => array(
			"friendly_name" => "Refresh Time",
			"description" => "How often to refresh in Cycle mode. Automatic makes all available maps fit into 5 minutes.",
			"method" => "drop_array",
			"array" => array(0 => "Automatic", 5 => "5 Seconds",
					15 => '15 Seconds',
					30 => '30 Seconds',
					60 => '1 Minute',
					120 => '2 Minutes',
					300 => '5 Minutes',
				)
			)
	    	);
	if (isset($settings["misc"]))
		$settings["misc"] = array_merge($settings["misc"], $temp);
	else
		$settings["misc"]=$temp;
}


function weathermap_setup_table () {
	global $config, $database_default;
	include_once($config["library_path"] . "/database.php");
	
	$sql = "show tables from " . $database_default;
	$result = db_fetch_assoc($sql) or die (mysql_error());
	
	$tables = array();
	$sql = array();

	foreach($result as $index => $arr) {
	  foreach ($arr as $t) {
	    $tables[] = $t;
	  }
	}
	
	$sql[] = "update weathermap_maps set sortorder=id where sortorder is null;";

	if (!in_array('weathermap_maps', $tables)) {
	  $sql[] = "CREATE TABLE weathermap_maps (
				  id int(11) NOT NULL auto_increment,
				  sortorder int(11) NOT NULL default 0,
				  active set('on','off') NOT NULL default 'on',
				  configfile text NOT NULL,
				  imagefile text NOT NULL,
				  htmlfile text NOT NULL,
				  titlecache text NOT NULL,
				  PRIMARY KEY  (id)
				) TYPE=MyISAM;";
	}
	else
	{
	  $colsql = "show columns from weathermap_maps from " . $database_default;
	  $result = mysql_query($colsql) or die (mysql_error());
	  $found = false;
	  while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
	    if ($row['Field'] == 'sortorder')
	      $found = true;
	  }
	  if (!$found)
	    {
	      $sql[] = "alter table weathermap_maps add sortorder int(11) NOT NULL default 0 after id";
	    }
	}

	if (!in_array('weathermap_auth', $tables)) {
	  $sql[] = "CREATE TABLE weathermap_auth (
					  userid mediumint(9) NOT NULL default '0',
					  mapid int(11) NOT NULL default '0'
					) TYPE=MyISAM;";
	}

	# create the settings entries, if necessary
	
	$pagestyle = read_config_option("weathermap_pagestyle");
	if($pagestyle == '' or $pagestyle < 0 or $pagestyle >1)
	  {
	    $sql[] = "replace into settings values('weathermap_pagestyle',0)";
	  }
	
	$cycledelay = read_config_option("weathermap_cycle_refresh");  
	if($cycledelay == '' or intval($cycledelay < 0) )
	  {
	    $sql[] = "replace into settings values('weathermap_cycle_refresh',0)";
	  }

	$tsize = read_config_option("weathermap_thumbsize");
	if($tsize == '' or $tsize < 1)
	  {
	    $sql[] = "replace into settings values('weathermap_thumbsize',250)";
	  }

	// patch up the sortorder for any maps that don't have one.
	$sql[] = "update weathermap_maps set sortorder=id where sortorder is null or sortorder=0;";

	if (!empty($sql)) {
	  for ($a = 0; $a < count($sql); $a++) {
	    $result = db_execute($sql[$a]);
	  }
	}
}

function weathermap_config_arrays () {
        global $user_auth_realms, $user_auth_realm_filenames, $menu;
		
        $user_auth_realms[42]='Configure Weathermap';
        $user_auth_realms[43]='View Weathermaps';
        $user_auth_realm_filenames['weathermap-cacti-plugin.php'] = 43;
        $user_auth_realm_filenames['weathermap-cacti-plugin-mgmt.php'] = 42;
		
	$menu["Management"]['plugins/weathermap/weathermap-cacti-plugin-mgmt.php'] = "Weathermaps";
}

function weathermap_show_tab () {
	global $config, $user_auth_realms, $user_auth_realm_filenames;
	$realm_id2 = 0;

        if (isset($user_auth_realm_filenames[basename('weathermap-cacti-plugin.php')])) {
                $realm_id2 = $user_auth_realm_filenames[basename('weathermap-cacti-plugin.php')];
        }

        if ((db_fetch_assoc("select user_auth_realm.realm_id from user_auth_realm where user_auth_realm.user_id='" . $_SESSION["sess_user_id"] . "' and user_auth_realm.realm_id='$realm_id2'")) || (empty($realm_id2))) {

		print '<a href="' . $config['url_path'] . 'plugins/weathermap/weathermap-cacti-plugin.php"><img src="' . $config['url_path'] . 'plugins/weathermap/images/tab_weathermap.png" alt="Weathermap" align="absmiddle" border="0"></a>';

        }

	weathermap_setup_table();
}

function weathermap_draw_navigation_text ($nav) {
   $nav["weathermap-cacti-plugin.php:"] = array("title" => "Weathermap", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin.php", "level" => "1");
   $nav["weathermap-cacti-plugin.php:viewmap"] = array("title" => "Weathermap", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin.php", "level" => "1");
   $nav["weathermap-cacti-plugin.php:viewmapcycle"] = array("title" => "Weathermap", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin.php", "level" => "1");

   $nav["weathermap-cacti-plugin-mgmt.php:"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
#   $nav["weathermap-cacti-plugin-mgmt.php:addmap_picker"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
   $nav["weathermap-cacti-plugin-mgmt.php:viewconfig"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
   $nav["weathermap-cacti-plugin-mgmt.php:addmap"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
   $nav["weathermap-cacti-plugin-mgmt.php:editmap"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
   $nav["weathermap-cacti-plugin-mgmt.php:editor"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");

#  "graphs.php:graph_edit" => array("title" => "(Edit)", "mapping" => "index.php:,graphs.php:", "url" => "", "level" => "2"),
 
   $nav["weathermap-cacti-plugin-mgmt.php:perms_edit"] = array("title" => "Edit Permissions", "mapping" => "index.php:,weathermap-cacti-plugin-mgmt.php:", "url" => "", "level" => "2");
   $nav["weathermap-cacti-plugin-mgmt.php:addmap_picker"] = array("title" => "Add Map", "mapping" => "index.php:,weathermap-cacti-plugin-mgmt.php:", "url" => "", "level" => "2");
  

   # $nav["weathermap-cacti-plugin-mgmt.php:perms_edit"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
     $nav["weathermap-cacti-plugin-mgmt.php:perms_add_user"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
	 $nav["weathermap-cacti-plugin-mgmt.php:perms_delete_user"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
   $nav["weathermap-cacti-plugin-mgmt.php:delete_map"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
  $nav["weathermap-cacti-plugin-mgmt.php:move_map_down"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
  $nav["weathermap-cacti-plugin-mgmt.php:move_map_up"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
   $nav["weathermap-cacti-plugin-mgmt.php:activate_map"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
   $nav["weathermap-cacti-plugin-mgmt.php:deactivate_map"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
   return $nav;
}

function weathermap_poller_bottom () {
        global $config;
	global $weathermap_debugging, $WEATHERMAP_VERSION;
	
	include_once($config["library_path"] . "/database.php");

	include_once(dirname(__FILE__)."/HTML_ImageMap.class.php");
	include_once(dirname(__FILE__)."/Weathermap.class.php");

        $outdir = dirname(__FILE__).'/output/';
        $confdir = dirname(__FILE__).'/configs/';

	if(module_checks())
	{
		weathermap_setup_table();
		
		// move to the weathermap folder so all those relatives paths don't *have* to be absolute
		$orig_cwd = getcwd();
		chdir(dirname(__FILE__));

		warn("Weathermap $WEATHERMAP_VERSION in da house");

		// first, see if the output directory even exists
		if(is_dir($outdir))
		{
			// next, make sure that we stand a chance of writing files
			# $testfile = realpath($outdir."weathermap.permissions.test");
			$testfile = $outdir."/weathermap.permissions.test";
			$testfd = fopen($testfile, 'w');
			if($testfd)
			{ 
				fclose($testfd); 
				unlink($testfile);
						
				// take our debugging cue from the poller - turn on Poller debugging to get weathermap debugging
				if (read_config_option("log_verbosity") >= POLLER_VERBOSITY_DEBUG)
				{
					$weathermap_debugging = TRUE;
					cacti_log("Engaging Weathermap DEBUG mode",true,"WEATHERMAP");
				}
				else
				{
					cacti_log("Normal logging mode. If things are going wrong, turn on DEBUG in Cacti for more information!", true, WEATHERMAP);
				}

				$queryrows = db_fetch_assoc("select * from weathermap_maps where active='on'");

				if( is_array($queryrows) )
				{
					debug("Iterating all maps.");
					$mapcount = 0;

					foreach ($queryrows as $map) {
						$mapfile = $confdir.'/'.$map['configfile'];
						$htmlfile = $outdir."/weathermap_".$map['id'].".html";
						$imagefile = $outdir."/weathermap_".$map['id'].".png";
						$thumbimagefile = $outdir."/weathermap_thumb_".$map['id'].".png";
						
						if(file_exists($mapfile))
						{
							warn("Map: $mapfile -> $htmlfile & $imagefile");
						
							$wmap = new Weathermap;
							
							$wmap->imageuri = 'output/weathermap_'.$map['id'].".png";
							// we can grab the rrdtool path from Cacti's config, in this case
							$wmap->rrdtool  = read_config_option("path_rrdtool");

							$wmap->ReadConfig($mapfile);	
							$wmap->ReadData();

							warn("About to write image file. If this is the last message in your log, increase memory_limit in php.ini");

							$wmap->DrawMap($imagefile,$thumbimagefile,read_config_option("weathermap_thumbsize"));
							
							warn("Wrote map to $imagefile and $thumbimagefile");
							$fd = fopen($htmlfile, 'w');
							fwrite($fd, $wmap->MakeHTML('weathermap_'.$map['id'].'_imap'));
							fclose($fd);
							debug("Wrote HTML to $htmlfile");

							db_execute("update weathermap_maps set titlecache='".mysql_real_escape_string($wmap->title)."' where id=".$map['id']);
							$mapcount++;
						}
						else
						{
							warn("Mapfile $mapfile is not readable or doesn't exist");
						}
					}
					debug("Iterated all $mapcount maps.");
				}
				else
				{
					warn("No activated maps found.");
				}
			}
			else
			{
				warn("Output directory ($outdir) isn't writable (tried to create '$testfile'). No maps created. You probably need to make it writable by the poller process (like you did with the RRA directory)");
			}
		}
		else
		{
			warn("Output directory ($outdir) doesn't exist!. No maps created. You probably need to create that directory, and make it writable by the poller process (like you did with the RRA directory)");
		}
		chdir($orig_cwd);
		warn("Weathermap has left the building");
	}
	else
	{
		warn("Required modules for PHP Weathermap were not present. Not running.");
	}
}


?>
